/*
 * $Id: ExceptionFactory.java,v 1.8 2009/12/30 11:25:21 oboehm Exp $
 *
 * Copyright (c) 2009 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * (c)reated 05.03.2009 by oliver (ob@aosd.de)
 */
package patterntesting.exception;

import java.lang.reflect.Constructor;

import javax.management.JMException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.impl.LogFactoryImpl;

import patterntesting.runtime.jmx.*;
import patterntesting.runtime.util.*;

/**
 * This class can create for testing purpose any desired checked exception.
 * Implemented with the help of Thomas.Darimont@web.de (via mailing list of
 * aspectj-users@eclipse.org).
 * <br/>
 * It is realized as Singleton to be able to extract an MBean interface for the
 * use via JMX.
 * <br/>
 * The exceptions are controlled by AbstractTestExceptionFactory.
 * You will see the ExceptionFactory not before the first
 * method marked as @TestException has finished (because it is realized as
 * after advice). If you want to see it before
 * call <tt>ExceptionFactory.getInstance()</tt> (with the creation of
 * the instance it is also registered as MBean).
 *
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 05.03.2009
 * @version $Revision: 1.8 $
 */
public final class ExceptionFactory implements ExceptionFactoryMBean {

    private static final Log log = LogFactoryImpl.getLog(ExceptionFactory.class);
    private static final ExceptionFactory instance;
    /** number of provoked exceptions */
    private long numberOfProvoked = 0L;
    /** maximal number of provoked exceptions */
    private long maxNumberOfProvoked = 0L;
	/** the last provoked exception which was thrown */
    private Throwable lastProvoked;
    /** only classes and subclasses of 'limitedTo' can be provoked */
    private Class<? extends Throwable> limitedTo = Throwable.class;

    static {
    	instance = new ExceptionFactory();
    	try {
    	    if (Assertions.enabled) {
    	        instance.registerAsMBean();
    	        if (log.isDebugEnabled()) {
    	            log.debug("ExceptionFactory registered as MBean");
    	        }
    	    } else {
    	        if (log.isDebugEnabled()) {
    	            log.debug("ExceptionFactory is disabled (no assertions)");
    	        }
    	    }
		} catch (JMException e) {
			log.info("can't register " + instance + " as MBean", e);
		}
    }

    private ExceptionFactory() {
    	if (log.isTraceEnabled()) {
    		log.trace(this + " created");
    	}
    }

    /**
     * We implement registerAsMBean ourself and not via MBeanRegistry
     * interface because we don't need the other methods declared in this
     * interface. And because the Ajdoc generation via maven failed because
     * it does not recognize patterntesting-rt as AspectJ lib.
     *
     * @throws JMException
     */
    private void registerAsMBean() throws JMException {
        MBeanHelper.registerMBean(this);
    }

    /**
     * It is realized as singleton because of the MBean interface.
     *
     * @return the only instance
     */
    public static ExceptionFactory getInstance() {
    	return instance;
    }

    /**
     * @see ExceptionFactoryMBean#getMaxNumberOfProvoked()
     */
    public long getMaxNumberOfProvoked() {
        return this.maxNumberOfProvoked;
    }

    /**
     * @see ExceptionFactoryMBean#getNumberOfProvoked()
     */
    public long getNumberOfProvoked() {
        return this.numberOfProvoked;
    }

    /**
     * @see ExceptionFactoryMBean#setMaxNumberOfProvoked(long)
     */
    public void setMaxNumberOfProvoked(long n) {
        this.maxNumberOfProvoked = n;
    }

    /**
     * @return true if active
	 * @see ExceptionFactoryMBean#isActive()
	 */
    public synchronized boolean isActive() {
		return this.maxNumberOfProvoked > this.numberOfProvoked;
	}

    /**
	 * @see ExceptionFactoryMBean#setActive(boolean)
	 */
    public synchronized void setActive(final boolean active) {
		this.maxNumberOfProvoked = active ? Long.MAX_VALUE
                : this.numberOfProvoked;
	}

    /**
	 * @see ExceptionFactoryMBean#activate()
	 */
    public synchronized void activate() {
		this.setActive(true);
	}

    /**
     * @see ExceptionFactoryMBean#activateOnce()
     */
    public void activateOnce() {
        this.maxNumberOfProvoked = this.numberOfProvoked + 1;
    }

    /**
	 * @see ExceptionFactoryMBean#deactivate()
	 */
    public synchronized void deactivate() {
		this.setActive(false);
	}

	/**
	 * @see ExceptionFactoryMBean#isAssertsEnabled()
	 */
	public boolean isAssertsEnabled() {
		return Assertions.enabled;
	}

	/**
	 * @see ExceptionFactoryMBean#getLastProvoked()
	 */
	public Throwable getLastProvoked() {
		return lastProvoked;
	}

	/**
	 * @see ExceptionFactoryMBean#getLimitedTo()
	 */
	public synchronized String getLimitedTo() {
		return limitedTo.getName();
	}

	/**
	 * You want to limit the provoked exceptions to IOException (or subclass)?
	 * Then give it as parameter to this setter method.
	 * </br>
	 * If you want to reset it and allow all Exceptions call
	 * <tt>setLimitedTo(Trowable.class)</tt>.
	 *
	 * @param limitedTo the limitedTo to set (e.g. IOException.class)
	 */
	public synchronized void setLimitedTo(final Class<? extends Throwable> limitedTo) {
		this.limitedTo = limitedTo;
	}

	/**
	 * @see ExceptionFactoryMBean#setLimitedTo(java.lang.String)
	 */
	@SuppressWarnings("unchecked")
	public void setLimitedTo(final String limitedTo) throws ClassNotFoundException {
		this.limitedTo = (Class<? extends Throwable>) Class.forName(limitedTo);
	}

	/**
	 * "Not limited" means, limitedTo is a super class of the given type.
	 * @param type
	 * @return
	 */
	private boolean isNotLimited(final Class<?> type) {
		if (type.equals(limitedTo)) {
			return true;
		}
		if (type.equals(Throwable.class)) {
			return false;
		}
		return isNotLimited(type.getSuperclass());
	}

    /**
     * Be careful - you can provoke any Exception with the method without the
     * need to declare it with a throws statement. For example
     * <pre>provoke(IOException.class)</pre>
     * would throw an IOException.
     * <br/>
     * You can only provoke an Exception if the activ flag is set.
     * <br/>
     * WARNING: If the desired exception can't be instantiated an
     * InstantiationException or IllegalAccessException may be thrown.
     * <br/>
     * WARNING(2): This method is not synchronized.
     *
     * @param type e.g. IOException.class
     */
    public void provoke(final Class<? extends Throwable> type) {
    	if (Assertions.enabled && this.isActive() && this.isNotLimited(type)) {
    		Throwable t;
	        try {
	        	t = create(type);
	        } catch (Exception ex) {
	            log.debug("can't create " + type, ex);
	            t = ex;
	        }
	        lastProvoked = t;
	        this.numberOfProvoked++;
	        Thrower.provoke(t);
    	} else {
    		if (log.isTraceEnabled()) {
    			log.trace("active flag not set or not a subclass of "
    					+ this.limitedTo + " -> no " + type + " thrown");
    		}
    	}
    }

    /**
     * This method throws the first given Throwable type. If this fails the
     * next element array is tried to be created as Throwable.
     * <br/>
     * You can only provoke an Exception if the active flag is set.
     * <br/>
     * WARNING: This method is not synchronized!
     *
     * @param types a class array with exception types
     */
    public void provokeOneOf(final Class<? extends Throwable>[] types) {
    	if (this.isActive()) {
    		Throwable t = null;
	        for (int i = 0; i < types.length; i++) {
	        	if (isNotLimited(types[i])) {
		            try {
		                t = create(types[i]);
		                break;
		            } catch (Exception ex) {
		                log.debug("can't create " + types[i], ex);
		                t = ex;
		            }
	        	}
	        }
	        lastProvoked = t;
	        this.numberOfProvoked++;
	        Thrower.provoke(t);
    	} else {
    		if (log.isTraceEnabled()) {
                log.trace("active flag not set or not a subclass of "
                        + this.limitedTo + " -> no "
                        + Converter.toString(types) + " thrown");
    		}
    	}
    }

    /**
     * Creates any desired exception you want.
     * If the desired exception can't be instantiated an
     * InstantiationException or IllegalAccessException may be thrown.
     *
     * @param type the exception class you want to be created
     *
     * @return the instantiated exception or the caught exception
     *
     * @throws IllegalAccessException the illegal access exception
     * @throws InstantiationException the instantiation exception
     */
    public Throwable create(final Class<? extends Throwable> type)
			throws InstantiationException, IllegalAccessException {
		try {
			Constructor<? extends Throwable> ctor = type
					.getConstructor(String.class);
			return ctor.newInstance("created by ExceptionFactory");
		} catch (Exception e) {
			if (log.isTraceEnabled()) {
				log.trace("can't call 'new " + type + "(String)'", e);
			}
			return type.newInstance();
		}
	}

    /**
     * The trick here is to use the constructor to throw any desired exception.
     * So you can throw any exception without the need to have it as throws
     * clause.
     *
     * @author <a href="boehm@javatux.de">oliver</a>
     * @since 06.03.2009class
     * @version $Revision: 1.8 $
     */
    static class Thrower {
        private static Throwable throwable;

        private Thrower() throws Throwable {
            throw throwable;
        }

        /**
         * Provoke an exception.
         *
         * @param t the Throwable which should be used as provoked exception.
         */
        public static void provoke(Throwable t) {
            throwable = t;
            try {
                Thrower.class.newInstance();
            } catch (InstantiationException unexpected) {
                log.debug("can't instantiate Thrower class", unexpected);
            } catch (IllegalAccessException unexpected) {
                log.debug("can't access Thrower constructor", unexpected);
            }
        }
    }

}
