/*
 * $Id: ExceptionFactoryMBean.java,v 1.4 2009/12/19 22:33:20 oboehm Exp $
 *
 * Copyright (c) 2009 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package patterntesting.exception;

/**
 * The Interface ExceptionFactoryMBean.
 */
public interface ExceptionFactoryMBean {

	/**
	 * Checks if is active.
	 *
	 * @return true, if is active
	 */
	public abstract boolean isActive();

	/**
	 * You can only provoke Exceptions if the active flag is set.
	 *
	 * @param activ the activ
	 */
	public abstract void setActive(boolean activ);

	/**
	 * You can only provoke Exceptions if the active flag is set.
	 * Call this method to set it.
	 */
	public abstract void activate();

	/**
	 * If you want to provoke an Exception only for the next time use this
	 * method.
	 */
	public abstract void activateOnce();

	/**
	 * You don't want to provoke an exception any longer? Then call this
	 * method.
	 */
	public abstract void deactivate();

	/**
	 * To see if the (Sun) Java-VM was called with the option "-ea"
	 * ("enable asserts") this getter shows it.
	 *
	 * @return true if asserts are enabled
	 */
	public abstract boolean isAssertsEnabled();

	/**
	 * Gets the last provoked.
	 *
	 * @return the last exception which was thrown by one of the provoke methods
	 */
	public abstract Throwable getLastProvoked();

	/**
	 * Only exceptions of this returned type (or subclasses of it) are thrown.
	 * The return value is a String because of the use as MBean (works better
	 * in the JConsole).
	 *
	 * @return the limitedTo (e.g. "java.lang.Throwable")
	 */
	public abstract String getLimitedTo();

	/**
	 * You want to limit the provoked exceptions to IOException (or subclass)?
	 * Then give it as parameter to this setter method.
	 * </br>
	 * If you want to reset it and allow all Exceptions call
	 * <tt>setLimitedTo("java.lang.Throwable")</tt>.
	 * <br>
	 * For better use with the JConsole the class could given as String.
	 * But don't forget to give the complete classname (with package) as
	 * String.
	 *
	 * @param limitedTo e.g. "java.io.IOException"
	 *
	 * @throws ClassNotFoundException the class not found exception
	 */
	public abstract void setLimitedTo(String limitedTo)
			throws ClassNotFoundException;

	/**
	 * Gets the number of provoked.
	 *
	 * @return the number of provoked exeptions
	 */
	public abstract long getNumberOfProvoked();

	/**
	 * Limit the number of provoked maximal total number of provoked exceptions
	 * to n.
	 *
	 * @param n the maximal number of provoked exceptions
	 * (default is Long.MAX_VALUE)
	 */
	public abstract void setMaxNumberOfProvoked(long n);

	/**
	 * Gets the max number of provoked.
	 *
	 * @return the maximal number of provoked exceptions
	 * (default is Long.MAX_VALUE)
	 */
	public abstract long getMaxNumberOfProvoked();

}

/*
 * $Log: ExceptionFactoryMBean.java,v $
 * Revision 1.4  2009/12/19 22:33:20  oboehm
 * trailing spaces removed
 *
 * Revision 1.3  2009/12/14 17:15:50  oboehm
 * trailing tabs removed
 *
 * Revision 1.2  2009/09/28 08:10:08  oboehm
 * javadocs completed with the help of JAutodoc
 *
 * Revision 1.1  2009/03/23 14:02:07  oboehm
 * initial setup
 *
 * $Source: /cvsroot/patterntesting/PatternTesting08/patterntesting-exception/src/main/java/patterntesting/exception/ExceptionFactoryMBean.java,v $
 */
